@@ -13,20 +13,25 @@
 #   License or the Artistic License, as specified in the Perl README file.
 #
 #   Author: Jochen Wiedmann
-#           Email: jochen.wiedmann at freenet.de
+#           Am Eisteich 9
+#           72555 Metzingen
+#           Germany
+# 
+#           Email: joe@ispsoft.de
+#           Phone: +49 7123 14881
 #
 
 use strict;
 
-use RPC::PlClient::Comm ();
-use Net::Daemon::Log ();
-use IO::Socket ();
+require RPC::PlServer::Comm;
+require Net::Daemon::Log;
+require IO::Socket;
 
 
 package RPC::PlClient;
 
-$RPC::PlClient::VERSION = '0.2021_01';
-@RPC::PlClient::ISA = qw(Net::Daemon::Log);
+$RPC::PlClient::VERSION = '0.2000';
+@RPC::PlClient::ISA = qw(RPC::PlServer::Comm Net::Daemon::Log);
 
 
 ############################################################################
@@ -47,7 +52,6 @@ sub new ($@) {
     my $self = {@_};
     bless($self, (ref($proto) || $proto));
 
-    my $comm = $self->{'comm'} = RPC::PlClient::Comm->new($self);
     my $app = $self->{'application'}  or
 	$self->Fatal("Missing application name");
     my $version = $self->{'version'}  or
@@ -59,8 +63,7 @@ sub new ($@) {
     if (!($socket = $self->{'socket'})) {
 	$self->Fatal("Missing peer address") unless $self->{'peeraddr'};
 	$self->Fatal("Missing peer port")
-	    unless ($self->{'peerport'}  ||
-		    index($self->{'peeraddr'}, ':') != -1);
+	    unless ($self->{'peerport'}  ||  $self->{'peeraddr'} =~ /:/);
 	$socket = $self->{'socket'} = IO::Socket::INET->new
 	    ('PeerAddr' => $self->{'peeraddr'},
 	     'PeerPort' => $self->{'peerport'},
@@ -73,11 +76,11 @@ sub new ($@) {
 		 $socket->peerhost(), $socket->peerport());
     $self->Debug("Sending login message: %s, %s, %s, %s",
 		 $app, $version, $user, "x" x length($password));
-    $comm->Write($socket, [$app, $version, $user, $password]);
+    $self->Write([$app, $version, $user, $password]);
     $self->Debug("Waiting for server's response ...");
-    my $reply = $comm->Read($socket);
-    die "Unexpected EOF from server" unless defined($reply);
-    die "Expected server to return an array ref" unless ref($reply) eq 'ARRAY';
+    my $reply = $self->Read();
+    die "Expected server to return an array ref"
+	unless $reply && ref($reply) eq 'ARRAY';
     my $msg = defined($reply->[1]) ? $reply->[1] : '';
     die "Refused by server: $msg" unless $reply->[0];
     $self->Debug("Logged in, server replies: $msg");
@@ -103,13 +106,15 @@ sub new ($@) {
 
 sub Call ($@) {
     my $self = shift;
-    my $socket = $self->{'socket'};
-    my $comm = $self->{'comm'};
-    $comm->Write($socket, [@_]);
-    my $msg = $comm->Read($socket);
-    die "Unexpected EOF while waiting for server reply" unless defined($msg);
-    die "Server returned error: $$msg" if ref($msg) eq 'SCALAR';
-    die "Expected server to return an array ref" unless ref($msg) eq 'ARRAY';
+    $self->Write([@_]);
+    my $msg = $self->Read();
+    if (!$msg  ||  !ref($msg)) {
+	die "Expected server to return a reference";
+    } elsif (ref($msg) eq 'SCALAR') {
+	die "Server returned error: $$msg";
+    } elsif (ref($msg) ne 'ARRAY') {
+	die "Expected server to return an array ref";
+    }
     @$msg;
 }
 
@@ -119,13 +124,8 @@ sub ClientObject {
     die "Constructor didn't return a TRUE value" unless $object;
     die "Constructor didn't return an object"
 	unless $object =~ /^((?:\w+|\:\:)+)=(\w+)/;
-    RPC::PlClient::Object->new($1, $client, $object);
-}
 
-sub Disconnect {
-    my $self = shift;
-    $self->{'socket'} = undef;
-    1;
+    RPC::PlClient::Object->new($1, $client, $object);
 }
 
 
@@ -135,11 +135,8 @@ use vars qw($AUTOLOAD);
 
 sub AUTOLOAD {
     my $method = $AUTOLOAD;
-    my $index;
-    die "Cannot parse method: $method"
-	unless ($index = rindex($method, '::')) != -1;
-    my $class = substr($method, 0, $index);
-    $method = substr($method, $index+2);
+    $method =~ s/(.*):://;
+    my $class = $1;
     eval <<"EOM";
         package $class;
         sub $method {
@@ -168,12 +165,10 @@ sub new {
 
 
 sub DESTROY {
-    my $saved_error = $@; # Save $@
     my $self = shift;
     if (my $client = delete $self->{'client'}) {
 	eval { $client->Call('DestroyHandle', $self->{'object'}) };
     }
-    $@ = $saved_error;    # Restore $@
 }
 
 1;
@@ -289,11 +284,6 @@ name, a protocol version and optionally a user name and password.
 These arguments are handled by the servers I<Application>, I<Version>
 and I<User> methods.
 
-=item compression
-
-Set this to off (default, no compression) or gzip (requires the
-Compress::Zlib module).
-
 =item cipher
 
 This attribute can be used to add encryption quite easily. PlRPC is not
@@ -309,16 +299,10 @@ that both sides change the encryption mode.
 Example:
 
     use Crypt::DES;
-    $cipher = Crypt::DES->new(pack("H*", "0123456789abcdef"));
+    $cipher = DES->new(pack("H*", "0123456789abcdef"));
     $client = RPC::PlClient->new('cipher' => $cipher,
 				...);
 
-=item maxmessage
-
-The size of messages exchanged between client and server is restricted,
-in order to omit denial of service attacks. By default the limit is
-65536 bytes.
-
 =item debug
 
 Enhances logging level by emitting debugging messages.
@@ -401,19 +385,18 @@ RPC::PlServer man page. See L<RPC::PlServer(3)>.
     require RPC::PlClient;
 
     # Constants
-    my $MY_APPLICATION = "MD5_Server";
+    my $MY_APPLICATION = "MD5 Server";
     my $MY_VERSION = 1.0;
     my $MY_USER = "";		# The server doesn't require user
     my $MY_PASSWORD = "";	# authentication.
 
     my $hexdigest = eval {
-        my $client = RPC::PlClient->new
-	    ('peeraddr'    => '127.0.0.1',
-	     'peerport'    => 2000,
-	     'application' => $MY_APPLICATION,
-	     'version'     => $MY_VERSION,
-	     'user'        => $MY_USER,
-	     'password'    => $MY_PASSWORD);
+        RPC::PlClient->new('peeraddr'    => 'joes.host.de',
+			   'peerport'    => 5000,
+			   'application' => $MY_APPLICATION,
+			   'version'     => $MY_VERSION,
+			   'user'        => $MY_USER,
+			   'password'    => $MY_PASSWORD);
 
         # Create an MD5 object on the server and an associated
         # client object. Executes a
@@ -440,7 +423,12 @@ RPC::PlServer man page. See L<RPC::PlServer(3)>.
 The PlRPC-modules are
 
   Copyright (C) 1998, Jochen Wiedmann
-                      Email: jochen.wiedmann at freenet.de
+                      Am Eisteich 9
+                      72555 Metzingen
+                      Germany
+
+                      Phone: +49 7123 14887
+                      Email: joe@ispsoft.de
 
   All rights reserved.
 
