@@ -11,7 +11,12 @@
 #   License or the Artistic License, as specified in the Perl README file.
 #
 #   Author: Jochen Wiedmann
-#           Email: jochen.wiedmann at freenet.de
+#           Am Eisteich 9
+#           72555 Metzingen
+#           Germany
+# 
+#           Email: joe@ispsoft.de
+#           Phone: +49 7123 14881
 #
 #
 
@@ -23,8 +28,8 @@ require RPC::PlServer::Comm;
 
 package RPC::PlServer;
 
-@RPC::PlServer::ISA = qw(Net::Daemon);
-$RPC::PlServer::VERSION = '0.2021_01';
+@RPC::PlServer::ISA = qw(Net::Daemon RPC::PlServer::Comm);
+$RPC::PlServer::VERSION = '0.2001';
 
 
 ############################################################################
@@ -44,37 +49,6 @@ sub Version ($) {
 }
 
 
-############################################################################
-#
-#   Name:    Options (Class method)
-#
-#   Purpose: Returns a hash ref of command line options
-#
-#   Inputs:  $class - This class
-#
-#   Result:  Options array; any option is represented by a hash ref;
-#            used keys are 'template', a string suitable for describing
-#            the option to Getopt::Long::GetOptions and 'description',
-#            a string for the Usage message
-#
-############################################################################
-
-sub Options ($) {
-    my $options = shift->SUPER::Options();
-    $options->{'maxmessage'} =
-	{ 'template' => 'maxmessage=i',
-	  'description' =>  '--maxmessage <size>           '
-	  . 'Set max message size to <size> (Default 65535).'
-	};
-    $options->{'compression'} =
-	{ 'template' => 'compression=s',
-	  'description' =>  '--compression <type>           '
-	  . 'Set compression type to off (default) or gzip.'
-	};
-    $options;
-}
-
-
 ############################################################################
 #
 #   Name:    AcceptApplication, AcceptVersion, AcceptUser
@@ -131,8 +105,6 @@ sub AcceptUser ($$$) {
 
 sub Accept ($) {
     my $self = shift;
-    my $socket = $self->{'socket'};
-    my $comm = $self->{'comm'};
     return 0 if (!$self->SUPER::Accept());
     my $client;
     if ($client = $self->{'client'}) {
@@ -142,9 +114,10 @@ sub Accept ($) {
 	}
     }
 
-    my $msg = $comm->Read($socket);
-    die "Unexpected EOF from client" unless defined $msg;
-    die "Login message: Expected array, got $msg" unless ref($msg) eq 'ARRAY';
+    my $msg = $self->Read();
+    if (ref($msg) ne 'ARRAY') {
+	die "Login message: Expected array, got $msg";
+    }
 
     my $app      = $self->{'application'} = $msg->[0] || '';
     my $version  = $self->{'version'}     = $msg->[1] || 0;
@@ -155,22 +128,19 @@ sub Accept ($) {
 		 $app, $version, $user);
 
     if (!$self->AcceptApplication($app)) {
-	$comm->Write($socket,
-		     [0, "This is a " . ref($self) . " server, go away!"]);
+	$self->Write([0, "This is a " . ref($self) . " server, go away!"]);
 	return 0;
     }
     if (!$self->AcceptVersion($version)) {
-	$comm->Write($socket,
-		     [0, "Sorry, but I am not running version $version."]);
+	$self->Write([0, "Sorry, but I am not running version $version."]);
 	return 0;
     }
     my $result;
     if (!($result = $self->AcceptUser($user, $password))) {
-	$comm->Write($socket,
-		     [0, "User $user is not permitted to connect."]);
+	$self->Write([0, "User $user is not permitted to connect."]);
 	return 0;
     }
-    $comm->Write($socket, (ref($result) ? $result : [1, "Welcome!"]));
+    $self->Write(ref($result) ? $result : [1, "Welcome!"]);
     if (my $au = $self->{'authorized_user'}) {
 	if (ref($au)  &&  (my $cipher = $au->{'cipher'})) {
 	    $self->Debug("User encryption: %s", $cipher);
@@ -193,27 +163,6 @@ sub Accept ($) {
 }
 
 
-############################################################################
-#
-#   Name:    new (Class method)
-#
-#   Purpose: Constructor
-#
-#   Inputs:  $class - This class
-#            $attr - Hash ref of attributes
-#            $args - Array ref of command line arguments
-#
-#   Result:  Server object for success, error message otherwise
-#
-############################################################################
-
-sub new ($$;$) {
-    my $self = shift->SUPER::new(@_);
-    $self->{'comm'} = RPC::PlServer::Comm->new($self);
-    $self;
-}
-
-
 ############################################################################
 #
 #   Name:    Run
@@ -228,15 +177,14 @@ sub new ($$;$) {
 
 sub Run ($) {
     my $self = shift;
-    my $comm = $self->{'comm'};
     my $socket = $self->{'socket'};
 
-    while (!$self->Done()) {
-	my $msg = $comm->Read($socket);
-	last unless defined($msg);
-	die "Expected array" unless ref($msg) eq 'ARRAY';
+    while (!$self->Done()  &&  !$socket->eof()) {
+	my $msg = $self->Read();
 	my($error, $command);
-	if (!($command = shift @$msg)) {
+	if (ref($msg) ne 'ARRAY') {
+	    $error = "Expected array";
+	} elsif (!($command = shift @$msg)) {
 	    $error = "Expected method name";
 	} else {
 	    if ($self->{'methods'}) {
@@ -252,12 +200,12 @@ sub Run ($) {
 		if ($@) {
 		    $error = "Failed to execute method $command: $@";
 		} else {
-		    $comm->Write($socket, \@result);
+		    $self->Write(\@result);
 		}
 	    }
 	}
 	if ($error) {
-	    $comm->Write($socket, \$error);
+	    $self->Write(\$error);
 	}
     }
 }
@@ -304,14 +252,7 @@ sub DestroyHandle ($$) {
 sub CallMethod ($$$@) {
     my($self, $handle, $method, @args) = @_;
     my($ref, $object);
-
-    my $call_by_instance;
-    {
-	my $lock = lock($Net::Daemon::RegExpLock)
-	    if $Net::Daemon::RegExpLock && $self->{'mode'} eq 'threads';
-	$call_by_instance = ($handle =~ /=\w+\(0x/);
-    }
-    if ($call_by_instance) {
+    if ($handle =~ /=\w+\(0x/) {
 	# Looks like a call by instance
 	$object = $self->UseHandle($handle);
 	$ref = ref($object);
@@ -395,35 +336,6 @@ The RPC::PlServer and RPC::PlClient are abstract servers and clients: You
 have to derive your own classes from it.
 
 
-=head2 Additional options
-
-The RPC::PlServer inherits all of Net::Daemon's options and attributes
-and adds the following:
-
-=over 8
-
-=item I<cipher>
-
-The attribute value is an instance of Crypt::DES, Crypt::IDEA or any
-other class with the same API for block encryption. If you supply
-such an attribute, the traffic between client and server will be
-encrypted using this option.
-
-=item I<maxmessage> (B<--maxmessage=size>)
-
-The size of messages exchanged between client and server is restricted,
-in order to omit denial of service attacks. By default the limit is
-65536 bytes.
-
-=item users
-
-This is an attribute of the client object used for Permit/Deny rules
-in the config file. It's value is an array ref of user names that
-are allowed to connect from the given client. See the example config
-file below. L<CONFIGURATION FILE>.
-
-=back
-
 =head2 Error Handling
 
 Error handling is simple with the RPC package, because it is based on
@@ -517,7 +429,7 @@ config file might look as follows:
         'user' => 'nobody',
         'group' => 'nobody',
         'localport' => '1003',
-        'mode' => 'fork',
+        'mode' => 'fork'
 
         # Access control
         'clients' => [
@@ -531,12 +443,12 @@ config file might look as follows:
             # Accept myhost.company.com
             {
                 'mask' => '^myhost\.company\.com$',
-                'accept' => 1,
+                'accept' => 1
                 'users' => [ {
                     'name' => 'bob',
                     'cipher' => $bob_key
                     } ]
-            },
+            }
             # Deny everything else
             {
                 'mask' => '.*',
@@ -575,12 +487,12 @@ is part of the RPC::PlClient man page. See L<RPC::PlClient(3)>.
 
     $MD5_Server::VERSION = '1.0'; # Clients will be refused, if they
                                   # request version 1.1
-    @MD5_Server::ISA = qw(RPC::PlServer);
+    @MD5_Server::ISA = qw(Net::Daemon);
 
     eval {
         # Server options below can be overwritten in the config file or
         # on the command line.
-        my $server = MD5_Server->new({
+        my $server = RPC::PlServer->new({
 	    'pidfile'    => '/var/run/md5serv.pid',
 	    'configfile' => '/etc/md5serv.conf',
 	    'facility'   => 'daemon', # Default
@@ -588,12 +500,11 @@ is part of the RPC::PlClient man page. See L<RPC::PlClient(3)>.
 	    'group'      => 'nobody',
 	    'localport'  => 2000,
 	    'logfile'    => 0,        # Use syslog
-            'mode'       => 'fork',   # Recommended for Unix
+            'mode'       => 'fork'    # Recommended for Unix
             'methods'    => {
 	        'MD5_Server' => {
 		    'ClientObject' => 1,
-		    'CallMethod' => 1,
-		    'NewHandle' => 1
+		    'CallMethod' => 1
 		    },
 	        'MD5' => {
 		    'new' => 1,
@@ -601,7 +512,7 @@ is part of the RPC::PlClient man page. See L<RPC::PlClient(3)>.
 		    'hexdigest' => 1
 		    },
 	        }
-        });
+        };
         $server->Bind();
     };
 
@@ -674,7 +585,12 @@ DBI::ProxyServer for an example.
 The PlRPC-modules are
 
   Copyright (C) 1998, Jochen Wiedmann
-                      Email: jochen.wiedmann at freenet.de
+                      Am Eisteich 9
+                      72555 Metzingen
+                      Germany
+
+                      Phone: +49 7123 14887
+                      Email: joe@ispsoft.de
 
   All rights reserved.
 
